
#include <RFM69.h>
#include <RFM69registers.h>
#include <RFM69_stream.h>
#include <IntervalTimer.h>

// TODO:
// 1) fix analog resolution to get max dynamic range
// 2) add low pass filter to DAC output
// 3) Use 3.3V regulator with higher output power

//NOTES:
// We transmit a 61-bit packet every 4880 us
// That mease we sample every 80us
// The packet transmission creates a tone of 200Hz
// The bit-rate results in a BW of 6.25 kHz
// Human speach ranges from 200Hz to 5kH.
// We can use a band-pass filter to cilter out 200Hz carrier and 6.25kHz DAC noise

#define NETWORKID     0   // Must be the same for all nodes
// For simplicity, both transmitter and receiver have the same ID
#define NODEID        2   // This doesn't reall matter because we don't check for it

// RFM69 frequency, uncomment the frequency of your module:
//#define FREQUENCY   RF69_433MHZ
#define FREQUENCY     RF69_915MHZ

#define ENCRYPTKEY    "TOPSECRETPASSWRD" // Use the same 16-byte key on all nodes
#define IS_RFM69HCW   true // set to 'true' if you are using an RFM69HCW module

#define RFM69_CS      10
#define RFM69_IRQ     2
#define RFM69_RST     9

#define BIT_RATE      80 // in micro-seconds

// Create a library object for our RFM69HCW module:
RFM69_stream radio = RFM69_stream(RFM69_CS, RFM69_IRQ, IS_RFM69HCW, RFM69_IRQ);

IntervalTimer myTimer;
unsigned long startTime;
unsigned long roundTripTime;
uint8_t timerRunning;
uint8_t cypher;
uint8_t radioMode;
enum radioModeEnum {
  M_RX, // receiver on, read to get packet
  M_STREAM_RX, // streaming audio receiver
  M_STREAM_TX, // streaming audio transmitter
  M_TX, // transmiting data packet
  M_TX_END //
  };
uint8_t prev;
const int audio =  A12;
const int PA_EN = 22;

void setup()   {               
  pinMode(PA_EN, OUTPUT);
  
  //analogWriteFrequency(audio, 375000); // 375 kHz carrier
  analogWriteResolution(8);
  Serial.begin(115200);
  //Serial1.begin(115200);
  delay(1000);
  Serial.println("init");

  // Hard Reset the RFM module
  pinMode(RFM69_RST, OUTPUT);
  digitalWrite(RFM69_RST, HIGH);
  delay(100);
  digitalWrite(RFM69_RST, LOW);
  delay(100);
  
  radio.initialize(FREQUENCY, NODEID, NETWORKID);
  radio.setHighPower(); // Always use this for RFM69HCW
  radio.setPowerLevel(31); // power output ranges from 0 (5dBm) to 31 (20dBm)
  radio.encrypt(ENCRYPTKEY);
  
  radio.dataRate200k();
  digitalWrite(PA_EN, 1);
  Serial.println("ready");

  timerRunning = 0;
  radioMode = M_RX;

}

void loop()                     
{
  //check if something was received (could be an interrupt from the radio)
  if ((radioMode == M_RX || radioMode == M_STREAM_RX) && radio.receiveDone())
  {
    // for printing dubug stuff
    printPacketDebug();

    // If the sender requested an ACK then the packet is for coms setup, not a stream buffer
    if (radioMode == M_RX)
    {
      //check if received message contains a Cypher
      if (strstr((char *)radio.DATA, "Cypher"))
      {
        cypher = *(radio.DATA+8);

        radio.sendACK();
        Serial.println("ACK sent");
        //delay(100);
        radio.encrypt(0);
        Serial.print("Cypher: ");
        Serial.println(cypher);
        //delay(1000);
        radioMode = M_STREAM_RX;
      }
    } else {
      // If no ack was requested it was a stream packet
      radio.receiveStreamBuffer();
      if(!timerRunning) {
        timerRunning = 1;
        radio.activeBuffer = 0;
        radio.bufferCounter = 0;
        myTimer.begin(getData, BIT_RATE);
      }
    }
  } else if(radioMode == M_STREAM_RX && !radio.isStreamConnected() ){
    // If we lost connection, kill the stream
    myTimer.end();
    timerRunning = 0;
    radio.activeBuffer = 0;
    radio.bufferCounter = 0;
    radioMode = M_RX;
    
    radio.encrypt(ENCRYPTKEY);
    Serial.println("Stream finished");
  }
}

void getData()
{
  uint8_t data;
  data = radio.receiveStreamByte();
  //Serial.println(data);
  analogWrite(audio, data);
}

void printPacketDebug() {
  roundTripTime = micros() - startTime;
  if(roundTripTime > 6000)
    Serial.println(roundTripTime);
  //Serial.print("   [RX_RSSI:");Serial.print(radio.RSSI);Serial.println("]");
  startTime = micros();
  //print message received to serial
  //Serial.print('[');Serial.print(radio.SENDERID);Serial.print("] ");
  //for (uint8_t i = 0; i < radio.DATALEN; i++)
    //Serial.println(radio.DATA[i]);
  //Serial.print("   [RX_RSSI:");Serial.print(radio.RSSI);Serial.println("]");
}

