#include <RFM69.h>
#include <RFM69registers.h>
#include <RFM69_stream.h>
#include <ADC.h>
#include <IntervalTimer.h>
#include "door.h"

#define NETWORKID     0   // Must be the same for all nodes
#define MYNODEID      0   // My node IDnn
#define TONODEID      2   // Destination node ID

// RFM69 frequency, uncomment the frequency of your module:
//#define FREQUENCY   RF69_433MHZ
#define FREQUENCY     RF69_915MHZ

#define ENCRYPTKEY    "TOPSECRETPASSWRD" // Use the same 16-byte key on all nodes
#define IS_RFM69HCW   true // set to 'true' if you are using an RFM69HCW module

#define RFM69_CS      10
#define RFM69_IRQ     2
#define RFM69_RST     9

#define BIT_RATE      80 // in micro-seconds

// Create a library object for our RFM69HCW module:
RFM69_stream radio = RFM69_stream(RFM69_CS, RFM69_IRQ, IS_RFM69HCW, RFM69_IRQ);

ADC *adc = new ADC(); // adc object

IntervalTimer timer0, timer1; // timers
IntervalTimer myTimer;
uint8_t sinetable [32];
uint32_t data;
const int audio =  A12;
const int MIC =  A1;
uint8_t timerRunning;

void setup()
{
  sinetable[0]=127;  // Put 32 step 8 bit sine table into array. 
  sinetable[1]=152; 
  sinetable[2]=176; 
  sinetable[3]=198; 
  sinetable[4]=217; 
  sinetable[5]=233; 
  sinetable[6]=245; 
  sinetable[7]=252; 
  sinetable[8]=254; 
  sinetable[9]=252; 
  sinetable[10]=245; 
  sinetable[11]=233; 
  sinetable[12]=217; 
  sinetable[13]=198; 
  sinetable[14]=176; 
  sinetable[15]=152; 
  sinetable[16]=128; 
  sinetable[17]=103; 
  sinetable[18]=79; 
  sinetable[19]=57; 
  sinetable[20]=38; 
  sinetable[21]=22; 
  sinetable[22]=10; 
  sinetable[23]=3; 
  sinetable[24]=0; 
  sinetable[25]=3; 
  sinetable[26]=10; 
  sinetable[27]=22; 
  sinetable[28]=38; 
  sinetable[29]=57; 
  sinetable[30]=79; 
  sinetable[31]=103; 
  Serial.begin(115200);
  analogWriteResolution(8);
  adc->setAveraging(16); // set number of averages (lower noise)
  adc->setResolution(12); // set bits of resolution
  adc->setReference(1, ADC_0);

  
  //
  // Hard Reset the RFM module
  //
  pinMode(RFM69_RST, OUTPUT);
  digitalWrite(RFM69_RST, HIGH);
  delay(100);
  digitalWrite(RFM69_RST, LOW);
  delay(100);
  
  radio.initialize(FREQUENCY, MYNODEID, NETWORKID);
  radio.setHighPower(); // Always use this for RFM69HCW
  radio.setPowerLevel(31); // power output ranges from 0 (5dBm) to 31 (20dBm)
  radio.encrypt(ENCRYPTKEY);
  radio.dataRate200k();
  
  char radiopacket[10] = "Cypher: ";
  radiopacket[8] = '6';
  radiopacket[9] = '\0';
  Serial.print("Sending setup packet...: ");
  Serial.println(radiopacket);
    
  while (!radio.sendWithRetry(TONODEID, radiopacket, 10)) { //target node Id, message as string or byte array, message length
    Serial.println("Waiting");
  }
  radio.encrypt(0);
  delay(1000);

  Serial.println("starting stream");

  timerRunning = 1;
  myTimer.begin(sendData, BIT_RATE);  // Add a byte to the buffer every 125us, Send a packet every 8 ms
  adc->enableInterrupts(ADC_0);
  data = 0;
}

void loop()                     
{
    if(timerRunning) {
      radio.sendStreamBuffer(TONODEID);
    } else {
      char radiopacket[10] = "Cypher: ";
      radiopacket[8] = '6';
      radiopacket[9] = '\0';
      Serial.println(radiopacket);
      while (!radio.sendWithRetry(TONODEID, radiopacket, 10)) { //target node Id, message as string or byte array, message length
        Serial.println("Waiting");
      }
      radio.encrypt(0);
      delay(1000);

      Serial.println("starting stream");

      timerRunning = 1;
      myTimer.begin(sendData, BIT_RATE);  // Add a byte to the buffer every 125us, Send a packet every 8 ms
      data = 0;
    }
}

void sendData()
{
    //adc->startSingleRead(MIC, ADC_0);
    
    /*if(data < sounddata_length*2) {
      radio.bufferStreamByte((pgm_read_byte(&sounddata_dataL[(data/2)])+128));
    } else {
    */
    //radio.bufferStreamByte((pgm_read_byte(&sounddata_dataL[(data)])+128));
    //radio.bufferStreamByte(128);
    radio.bufferStreamByte(sinetable[(data)%32]);
    //Serial.println(sinetable[data%32]);
    data+=1;
    if(data >= sounddata_length*20) {
      myTimer.end();
      radio.encrypt(ENCRYPTKEY);
      timerRunning = 0;
    }
    
}

// when the measurement finishes, this will be called
void adc0_isr() {
  int16_t adc_data = adc->readSingle();
  adc_data -= 1280; //subtract DC
  if(adc_data > 127) {
    adc_data = 127;
  } else if(adc_data < -127) {
    adc_data = -127;
  }
  //Serial.println(adc_data + 128);
  radio.bufferStreamByte((uint8_t)((adc_data + 128)));
  //Serial.println(adc_data);

  // restore ADC config if it was in use before being interrupted by the analog timer
  if (adc->adc0->adcWasInUse) {
    // restore ADC config, and restart conversion
    adc->adc0->loadConfig(&adc->adc0->adc_config);
    // avoid a conversion started by this isr to repeat itself
    adc->adc0->adcWasInUse = false;
  }
}

